# coding: utf-8

import logging.config
import os
import time

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

timestr = time.strftime("%Y.%m.%d-%H:%M:%S")
tmp_file_name = '/tmp/tnx-autosetup-{0}.log'.format(timestr)


class AutosetupStreamHandler(logging.StreamHandler):
    def handle(self, record):
        original_msg = record.msg
        if hasattr(record, 'safe_msg'):
            record.msg = record.safe_msg
        try:
            return super(AutosetupStreamHandler, self).handle(record)
        finally:
            record.msg = original_msg


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'default': {
            'format': '%(asctime)s.%(msecs)03d %(process)d %(levelname)s '
                      '%(name)s [-] %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S',
        },
        'extended': {
            'format': '%(asctime)s - %(levelname)s - File: %(filename)s - '
                      '%(funcName)s() - Line: %(lineno)d - %(message)s'
        }
    },
    'handlers': {
        'console': {
            'class': 'tionix_autosetup.AutosetupStreamHandler',
            'formatter': 'default',
        },
        'tmp_file': {
            'class': 'logging.FileHandler',
            'formatter': 'default',
            'filename': tmp_file_name,
        },
    },
    'loggers': {
        'root': {
            'handlers': ['console'],
        },
        'tionix_autosetup': {
            'handlers': ['console', 'tmp_file'],
        },
    }
}


logging.config.dictConfig(LOGGING)
