# coding: utf-8

import logging
import subprocess

from tionix_autosetup.executors.base import (
    Executor, ModuleInstallCheckMixin, ModuleUpdateCheckMixin,
    ModuleUninstallCheckMixin)
from tionix_autosetup.helpers import PackageTypesEnum, ManagerTypesEnum

logger = logging.getLogger(__name__)


class RPMExecutor(Executor):
    packages = None
    command_templates = None
    package_type = PackageTypesEnum.RPM

    def __init__(self, *args, **kwargs):
        super(RPMExecutor, self).__init__(*args, **kwargs)
        self.packages = kwargs.get('packages', self.packages) or []
        self.package_manager = kwargs.get('package_manager',
                                          ManagerTypesEnum.APT)
        if not isinstance(self.packages, (list, tuple)):
            self.packages = [self.packages]

    @property
    def commands(self):
        commands_list = [
            self.command_templates.get(self.package_manager) % package
            for package in self.packages if package]
        return commands_list

    @staticmethod
    def _check_return_code(cmd):
        result = False
        try:
            p = subprocess.Popen(cmd, shell=True,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT)
            p.communicate()
            result = not bool(p.returncode)
        except OSError:
            pass
        return result

    def pre_check(self, *arg, **kwargs):
        which_cmd = 'which %s' % self.package_manager
        return self._check_return_code(which_cmd)

    def is_package_installed(self, package_name):
        rpm_cmd = 'dpkg -l {0} || rpm -q {0}'.format(package_name)
        return self._check_return_code(rpm_cmd)


class RPMInstall(ModuleInstallCheckMixin, RPMExecutor):
    critical_operation = True

    command_templates = {
        ManagerTypesEnum.APT: 'apt-get -y install %s',
    }


class RPMUpdate(ModuleUpdateCheckMixin, RPMExecutor):
    critical_operation = True

    command_templates = {
        ManagerTypesEnum.APT: 'apt-get -y install %s',
    }


class RPMUninstall(ModuleUninstallCheckMixin, RPMExecutor):
    critical_operation = True

    command_templates = {
        ManagerTypesEnum.APT: 'apt-get remove -y %s',
    }
