# coding: utf-8

from __future__ import unicode_literals

from tionix_autosetup.executors.base import (
    Executor, BaseConnectionCheckExecutor, ConnectionCheckException,
    HIDDEN_PASSWORD)


class MysqlMixin(object):
    access_denied = 'Access denied'
    unknown_database = 'Unknown database'
    cant_connect = "Can't connect"

    mysql_connect_cmd = ('mysql --user={DB_SUPERUSER_NAME} '
                         '--password={DB_SUPERUSER_PASSWORD} '
                         '--host={DB_HOST}')
    mysql_exec_cmd = mysql_connect_cmd + ' -e "%s"'
    mysql_check_cmd = mysql_connect_cmd + ' -D %s'


class DbConnectionCheck(MysqlMixin, BaseConnectionCheckExecutor):
    name = 'Mysql'

    def check_connection(self, var_dict):
        user = var_dict.get('DB_SUPERUSER_NAME')
        password = var_dict.get('DB_SUPERUSER_PASSWORD')
        db_host = var_dict.get('DB_HOST')

        command = 'SHOW DATABASES;'
        cmd = self.mysql_exec_cmd.format(DB_SUPERUSER_NAME=user,
                                         DB_SUPERUSER_PASSWORD=password,
                                         DB_HOST=db_host)

        safe_cmd = self.mysql_exec_cmd.format(
            DB_SUPERUSER_NAME=user, DB_HOST=db_host,
            DB_SUPERUSER_PASSWORD=HIDDEN_PASSWORD)

        result = self.sh_exec(cmd % command, safe_cmd % command)
        if self.access_denied in result or self.cant_connect in result:
            raise ConnectionCheckException(result)


class BaseMysqlCommands(MysqlMixin, Executor):
    """ Base Mysql class"""

    mysql_commands = ()

    @property
    def commands(self):
        return [self.mysql_exec_cmd % command
                for command in self.mysql_commands]

    def database_check(self):
        """Check if user and database is exists"""
        user = self.variables_dict.get('DB_SUPERUSER_NAME')
        password = self.variables_dict.get('DB_SUPERUSER_PASSWORD')
        db_name = self.variables_dict.get('DB_NAME')
        db_host = self.variables_dict.get('DB_HOST')

        cmd = self.mysql_check_cmd.format(DB_SUPERUSER_NAME=user,
                                          DB_SUPERUSER_PASSWORD=password,
                                          DB_HOST=db_host)

        safe_cmd = self.mysql_check_cmd.format(
            DB_SUPERUSER_NAME=user, DB_HOST=db_host,
            DB_SUPERUSER_PASSWORD=HIDDEN_PASSWORD)

        return self.sh_exec(cmd % db_name, safe_cmd % db_name)


class MysqlDropCommands(BaseMysqlCommands):
    """ Drop DATABASE """
    mysql_commands = (
        "DROP DATABASE {DB_NAME};",
    )

    def pre_check(self, *arg, **kwargs):
        """Check if database is exists"""
        if not self.force:
            return False
        else:
            result = self.database_check()
            return not (self.access_denied in result or
                        self.unknown_database in result)

    def post_check(self, *arg, **kwargs):
        return self.unknown_database in self.database_check()


class MysqlCommands(BaseMysqlCommands):
    """ Create DATABASE \ Create USER \ Grant permissions """

    critical_operation = True

    @property
    def mysql_commands(self):
        base_commands = (
            "CREATE DATABASE {DB_NAME};",
        )
        allowed_hosts = set(self.variables_dict.get('DB_ALLOWED_HOSTS'))
        db_host = self.variables_dict.get('DB_HOST')
        allowed_hosts.update(('localhost', db_host))

        privileges = tuple("GRANT ALL PRIVILEGES ON {DB_NAME}.* TO '" +
                           "{DB_USERNAME}'@'" + host + "' IDENTIFIED BY '" +
                           "{DB_USER_PASSWORD}';" for host in allowed_hosts)

        return base_commands + privileges

    def post_check(self, *arg, **kwargs):
        result = self.database_check()
        return not (self.access_denied in result or
                    self.unknown_database in result or
                    self.cant_connect in result)
