# coding: utf-8

from collections import namedtuple
import logging

from tionix_autosetup.executors.base import Executor
from tionix_autosetup.executors.pip_service import (
    PIPInstall, PIPUninstall, PIPUpdate)
from tionix_autosetup.executors import manager
from tionix_autosetup.helpers import (
    OperationsEnum, ManagerTypesEnum, PackageTypesEnum)

logger = logging.getLogger(__name__)

Strategy = namedtuple('Strategy', 'install update uninstall')


DEFAULT_SOURCE = ManagerTypesEnum.PIP
RPM_MANAGERS = (ManagerTypesEnum.APT, )


INSTALLATION_STRATEGIES = {
    PackageTypesEnum.PIP: Strategy(install=PIPInstall,
                                   update=PIPUpdate,
                                   uninstall=PIPUninstall),
    PackageTypesEnum.RPM: Strategy(install=manager.RPMInstall,
                                   update=manager.RPMUpdate,
                                   uninstall=manager.RPMUninstall),
}


class Install(Executor):

    action = OperationsEnum.INSTALL

    critical_operation = None
    default_manager = strategy = None
    packages = package_names = None

    def __init__(self, variable_box=None, **kwargs):
        super(Install, self).__init__(variable_box=variable_box, **kwargs)

        package_manager = variable_box.kwargs.get('source', DEFAULT_SOURCE)
        if package_manager == DEFAULT_SOURCE and self.default_manager:
            package_manager = self.default_manager

        package_type = PackageTypesEnum.PIP
        if package_manager in RPM_MANAGERS:
            package_type = PackageTypesEnum.RPM
        strategy_type = INSTALLATION_STRATEGIES.get(package_type)
        strategy_class = getattr(strategy_type, self.action)
        if not self.packages and self.package_names:
            packages = self.package_names.get(package_type)
            if isinstance(packages, dict):
                self.packages = packages.get(self.action)
            else:
                self.packages = packages
        kwargs['packages'] = self.packages
        kwargs['package_manager'] = package_manager
        self.strategy = strategy_class(variable_box=variable_box, **kwargs)
        if self.critical_operation is None:
            self.critical_operation = self.strategy.critical_operation

    @property
    def commands(self):
        return self.strategy.commands

    def run(self, **kwargs):
        return self.strategy.run(**kwargs)


class Update(Install):
    action = OperationsEnum.UPDATE


class Uninstall(Install):
    action = OperationsEnum.UNINSTALL
