# coding: utf-8

from __future__ import unicode_literals

import logging
import pkg_resources

from tionix_autosetup.executors.base import (
    Executor, ModuleInstallCheckMixin, ModuleUpdateCheckMixin,
    ModuleUninstallCheckMixin)
from tionix_autosetup.helpers import PackageTypesEnum

logger = logging.getLogger(__name__)


class PIPExecutor(Executor):
    package_type = PackageTypesEnum.PIP
    packages = None
    command_template = None

    update_pip_command = 'pip install -U pip'
    _updated = False

    def __init__(self, *args, **kwargs):
        super(PIPExecutor, self).__init__(*args, **kwargs)
        self.packages = kwargs.get('packages', self.packages) or []
        if not isinstance(self.packages, (list, tuple)):
            self.packages = [self.packages]

    @property
    def updated(self):
        return PIPInstall._updated

    @updated.setter
    def updated(self, value):
        PIPInstall._updated = value

    @property
    def commands(self):
        commands_list = [
            self.command_template % package
            for package in self.packages if package]
        if not self.updated:
            commands_list.insert(0, self.update_pip_command)
            self.updated = True
        return commands_list

    def pre_check(self, *arg, **kwargs):
        return self.is_package_installed('pip')

    @staticmethod
    def is_package_installed(package_name):
        # seperate package name from pip arguments if present
        package_name = package_name.split(' ')[-1]
        reload(pkg_resources)
        result = True
        try:
            package = pkg_resources.Requirement.parse(package_name)
        except (ValueError, pkg_resources.RequirementParseError):
            result = False

        else:
            package_name = package.project_name
            try:
                pkg_resources.get_distribution(package_name)
            except pkg_resources.DistributionNotFound:
                result = False
            except (pkg_resources.VersionConflict,
                    pkg_resources.ContextualVersionConflict) as err:
                logger.warning('An already-installed version conflicts with '
                               'the requested version: {0}'.format(err))
        return result


class PIPInstall(ModuleInstallCheckMixin, PIPExecutor):
    critical_operation = True

    command_template = (
        'export PIP_EXTRA_INDEX_URL=http://{PYPI_HOST}/simple/ && '
        'pip --no-cache-dir install {PIP_ARGS} %s '
        '--trusted-host {PYPI_HOST}')


class PIPUpdate(ModuleUpdateCheckMixin, PIPExecutor):
    critical_operation = True

    command_template = (
        'export PIP_EXTRA_INDEX_URL=http://{PYPI_HOST}/simple/ && '
        'pip install --no-cache-dir -U --no-deps {PIP_ARGS} %s'
        ' --trusted-host {PYPI_HOST}')


class PIPUninstall(ModuleUninstallCheckMixin, PIPExecutor):
    critical_operation = True

    command_template = 'pip uninstall -y %s'
