# coding: utf-8

from __future__ import unicode_literals

import shlex
import logging
import subprocess

from tionix_autosetup.executors.base import (
    Executor, BaseConnectionCheckExecutor)
from tionix_autosetup.helpers import installed_tionix_modules

logger = logging.getLogger(__name__)


class RabbitMQConnectionCheck(BaseConnectionCheckExecutor):
    name = 'RabbitMQ'

    def check_connection(self, var_dict):
        logger.info('Executing command: rabbitmq connection')

        host = var_dict.get('RABBIT_HOST')
        node_name = 'rabbit@%s' % host
        rabbit_cmd = 'rabbitmqctl -n %s status' % node_name

        subprocess.check_call(shlex.split(rabbit_cmd), stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE, stderr=subprocess.STDOUT)


class RabbitCommands(Executor):
    commands = (
        'rabbitmqctl add_user {RABBIT_USERNAME} {RABBIT_PASSWORD}',
        'rabbitmqctl add_vhost {RABBIT_VIRTHOST}',
        'rabbitmqctl set_permissions -p {RABBIT_VIRTHOST} {RABBIT_USERNAME} '
        '".*" ".*" ".*"',
        'rabbitmqctl set_permissions {RABBIT_USERNAME} ".*" ".*" ".*"',
    )

    @staticmethod
    def rabbit_user_exist(user):
        cmd = "rabbitmqctl list_users"
        p = subprocess.Popen(cmd, shell=True,
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
        result = p.communicate()[0]
        if 'Listing users' not in result:
            logger.warning(
                "Couldn't check rabbitmq user existence "
                "for user: {0}".format(user))
            return False

        users = (line.split()[0] for line in result.splitlines()[1:])
        if user not in users:
            return False
        return True

    def post_check(self, *arg, **kwargs):
        """ Check if RabbitMQ user exists after run """
        user = self.variables_dict.get('RABBIT_USERNAME')
        if not self.rabbit_user_exist(user):
            logger.critical("RabbitMQ user doesn't exist: {0}".format(
                user))
            return False
        return True


class DropRabbitUser(RabbitCommands):
    commands = (
        'rabbitmqctl delete_user {RABBIT_USERNAME}',
    )

    def pre_check(self, *arg, **kwargs):
        return self.force and not installed_tionix_modules()


class DropRabbitVhost(RabbitCommands):
    commands = (
        'rabbitmqctl delete_vhost {RABBIT_VIRTHOST}',
    )
    check_str = '{RABBIT_VIRTHOST}'
    check_cmd = "rabbitmqctl list_vhosts"

    def rabbit_vhost_exist(self):
        cmd_result = self.sh_exec(self.check_cmd)
        return (isinstance(cmd_result, basestring) and
                self.check_str.format(**self.variables_dict) in cmd_result)

    def pre_check(self, *arg, **kwargs):
        return self.force and self.rabbit_vhost_exist()
