# coding: utf-8

from __future__ import unicode_literals

import logging
import os

from tionix_autosetup.executors.base import Executor
from tionix_autosetup.helpers import (
    AUTOSETUP_CONF_DIR, save_installed_packages_info
)

logger = logging.getLogger(__name__)


class RollbackExecutor(Executor):

    PACKAGES_TO_DELETE = '{0}packages_to_delete.env'.format(AUTOSETUP_CONF_DIR)

    PACKAGES_TO_INSTALL = '{0}packages_to_install.env'.format(
        AUTOSETUP_CONF_DIR)

    def __init__(self, variable_box=None, **kwargs):
        super(RollbackExecutor, self).__init__(variable_box, **kwargs)

        self.pip_packages_actual = save_installed_packages_info()

        if hasattr(variable_box, 'kwargs'):
            self.pip_packages_before = variable_box.kwargs.get('file_path')

    def pre_check(self, *arg, **kwargs):
        result = True
        if (self.pip_packages_before and
                not os.path.exists(self.pip_packages_before)):
            logger.error('File with name {0} '
                         'not found'.format(self.pip_packages_before))
            result = False
        return result

    @property
    def commands(self):
        if not self.pip_packages_before:
            pip_packages_before_list = [os.path.join(AUTOSETUP_CONF_DIR, f)
                                        for f in os.listdir(AUTOSETUP_CONF_DIR)
                                        if f.lower().endswith('.env') and
                                        'pip_packages_before' in f.lower()]

            self.pip_packages_before = min(pip_packages_before_list,
                                           key=os.path.getctime)

        params = {
            'env_before': self.pip_packages_before,
            'env_actual': self.pip_packages_actual,
            'env_to_del': self.PACKAGES_TO_DELETE,
            'env_to_install': self.PACKAGES_TO_INSTALL,
        }

        return (
            ("diff --changed-group-format='%>' --unchanged-group-format='' "
             "{env_before} {env_actual} > {env_to_del}").format(**params),

            ("diff --changed-group-format='%<' --unchanged-group-format='' "
             "{env_before} {env_actual} > {env_to_install}").format(**params),

            ("export PIP_EXTRA_INDEX_URL=http://{PYPI_HOST}/simple/ && "
             "pip uninstall -y --no-cache-dir -r " + params.get('env_to_del') +
             " --trusted-host {PYPI_HOST}"),

            ("export PIP_EXTRA_INDEX_URL=http://{PYPI_HOST}/simple/ && "
             "pip install --no-cache-dir -r " + params.get('env_to_install') +
             " --trusted-host {PYPI_HOST}"),
        )
