# coding: utf-8

from __future__ import unicode_literals

import pwd

from tionix_autosetup.executors.base import Executor


class SystemServicesRestart(Executor):
    commands = (
        ('service nova-api restart', 'systemctl restart openstack-nova-api'),
        (
            'service cinder-api restart',
            'systemctl restart openstack-cinder-api'
        ),
        (
            'service heat-api restart',
            'systemctl restart openstack-heat-api'
        ),
        'supervisorctl update'
    )


class MemcachedRestart(Executor):
    commands = (
        ('service memcached restart', 'systemctl restart memcached'),
    )


class SupervisorStart(Executor):
    commands = (
        'systemctl enable supervisord.service',
        (
            'service supervisor start',
            'systemctl start supervisord',
            'supervisord',
        ),
    )


class SupervisorRestart(Executor):
    commands_template = 'supervisorctl restart {0}'


class AddUser(Executor):

    critical_operation = True

    commands = (
        'useradd -m {TNX_USERNAME}',
    )

    @staticmethod
    def _user_exist(username):
        result = True
        try:
            pwd.getpwnam(username)
        except KeyError:
            result = False
        return result

    @property
    def username(self):
        return self.variables_dict.get('TNX_USERNAME')

    def pre_check(self, *arg, **kwargs):
        """ Check if user exists before run

        Check passed if doesn't exist
        """
        return not self._user_exist(self.username)

    def post_check(self, *arg, **kwargs):
        """ Check if user exists after run

        Check passed if user exists
        """
        return self._user_exist(self.username)


class Migration(Executor):
    critical_operation = True

    module_name = None
    command_template = 'openstack tnx db migrate -n {module_name}'

    @property
    def commands(self):
        return (
            self.command_template.format(module_name=self.module_name),
        )


class RunConfiguration(Executor):
    critical_operation = True

    module_names = None
    command_template = 'openstack tnx configure -n {module_names}'

    @property
    def commands(self):
        names = self.module_names or ''
        if isinstance(names, (tuple, list)):
            names = ' '.join(names)

        return (
            self.command_template.format(module_names=names),
        )
