# coding: utf-8

import subprocess

from tionix_autosetup.table import build_wrapped_table


class ConnectionCheckResult(object):
    def __init__(self, service_name, result_code, result_message):
        self.service_name = service_name
        self.result_code = result_code
        self.result_message = result_message


def get_result_code(code):
    return u'OK' if code else u'FAIL'


def get_terminal_width():
    width = 80

    try:
        ouput = subprocess.check_output(['stty', 'size'], shell=True).split()
    except subprocess.CalledProcessError:
        pass
    else:
        if len(ouput) > 1:
            try:
                width = int(ouput[1])
            except ValueError:
                pass
    return width


def make_table(results):
    headers = [u'N', u'Service name', u'Status', u'Error message']
    rows = [[i, result.service_name, get_result_code(result.result_code),
             result.result_message] for i, result in enumerate(results, 1)]

    screen_width = get_terminal_width()
    formatted_table = build_wrapped_table(headers, rows, screen_width)
    return str(formatted_table) + '\n'


class ConnectionsCheckParser(object):
    max_details_length = 70

    def __init__(self):
        self.results = []
        self.connections_check_table = []

    @property
    def was_successful(self):
        return all([result.result_code for result in self.results])

    def add_result(self, command_exec, result, module_name):
        result_code = result.run_check
        if not result_code:
            result_msg = result.run_result
            service_name = "{0} ({1})".format(command_exec.name, module_name)
            self.results.append(ConnectionCheckResult(service_name,
                                                      result_code, result_msg))

    def get_results(self):
        table_data = make_table(self.results)
        table_name = u'Services connections table'
        self.connections_check_table = (
            u"{border}\n"
            u"{table_name}\n"
            u"{border}\n"
            u"{table_data}"
        ).format(
            table_name=table_name,
            border="=" * len(table_name),
            table_data=table_data
        )
        return self.connections_check_table
