# coding: utf-8

import itertools

import prettytable


MIN_ADJUSTABLE_WIDTH = 5
PADDING = 1


def adjust_columns_width(headers, rows, diff):
    columns_width = [0] * len(headers)
    for row in itertools.chain([headers], rows):
        for col_num, cell in enumerate(row):
            cell_width = max(map(len, str(cell).splitlines() + ['']))
            columns_width[col_num] = max(columns_width[col_num], cell_width)

    for i, width in reversed(list(enumerate(columns_width))):
        if width > MIN_ADJUSTABLE_WIDTH:
            new_width = max(width - diff, MIN_ADJUSTABLE_WIDTH)
            diff -= width - new_width
            columns_width[i] = new_width

    return columns_width


def build_table(headers, rows):
    table = prettytable.PrettyTable(headers)
    # prettytable doesn't support linebreaks in headers, so we add headers
    # to rows and print table without headers
    table.add_row(headers)
    for row in rows:
        table.add_row(row)
    table.header = False
    table.align = 'l'
    table.hrules = prettytable.ALL
    table.left_padding_width = table.right_padding_width = PADDING
    return table


def build_wrapped_table(headers, rows, max_width=None):
    table = build_table(headers, rows)
    table_width = len(str(table).splitlines()[0])

    if max_width and max_width < table_width:
        table.left_padding_width = table.right_padding_width = 0
        diff = table_width - max_width - PADDING * len(headers) * 2
        if diff > 0:
            columns_width = adjust_columns_width(headers, rows, diff)
            for col_num, header in enumerate(headers):
                table.max_width[header] = columns_width[col_num]
    return table
