# coding: utf-8

from __future__ import unicode_literals

import logging
import os

from tionix_autosetup.executors.base import AppendPath, MODULES_CONF_DIR, Dirs
from tionix_autosetup.executors.config import (
    DropPath, DropConfig, RenameConfig, DropSupervisorConfig,
    ConfigWithSectionsParser, Config, UpdateYamlConfig, ActualizeConfig)
from tionix_autosetup.executors.mysql import (
    DbConnectionCheck, MysqlDropCommands, MysqlCommands)
from tionix_autosetup.executors.rabbit import (
    RabbitMQConnectionCheck, RabbitCommands, DropRabbitVhost)
from tionix_autosetup.executors.keystone import KeystoneConnectionCheck
from tionix_autosetup.executors.package import (
    Install, Update, Uninstall)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, Migration, RunConfiguration, SupervisorRestart)
from tionix_autosetup.helpers import PackageTypesEnum


logger = logging.getLogger(__name__)
dirs = ['/var/log/tionix/cloud-control']
config_name = 'cloud_control.yaml'
module_config_prefix = 'tnx_cc_'
supervisor_process = 'tionix-cc-worker'


package_names = {
    PackageTypesEnum.PIP: 'tionix-cloud-control',
    PackageTypesEnum.RPM: 'python-module-tionix_cloud_control',
}


class CloudControlMigration(Migration):
    module_name = 'tnx_cloud_control'


class CloudControlConfiguration(RunConfiguration):
    module_names = 'tnx_cloud_control'


class CloudControlConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, config_name)
    template_name = 'cloud_control_config.template'


class UpdateCloudControlYamlConfig(UpdateYamlConfig, CloudControlConfig):
    pass


class ConfigWithSectionsParserCCWorker(ConfigWithSectionsParser):
    config_path = '/etc/supervisor/conf.d/tnx_cc_worker.conf'
    config_section = 'program:{0}'.format(supervisor_process)
    config_keys = {
        'user': '{TNX_USERNAME}',
        'group': '{TNX_USERNAME}',
    }


class CloudControlDirs(Dirs):
    dirs = dirs


class CloudControlDropConfig(DropConfig):
    config_name = config_name


class CloudControlActualizeConfig(ActualizeConfig):
    config_name = config_name


class CloudControlDropSupervisorConfig(DropSupervisorConfig):
    module_config_prefix = module_config_prefix


class CloudControlRenameConfig(RenameConfig):
    config_name = config_name


class CloudControlDropLogFiles(DropPath):
    paths = dirs


class InstallCC(Install):
    package_names = package_names


class UpdateCC(Update):
    package_names = package_names


class UninstallCC(Uninstall):
    package_names = package_names


class SupervisorRestartCC(SupervisorRestart):

    @property
    def commands(self):
        return [self.commands_template.format(supervisor_process)]


pipeline = [
    InstallCC,
    CloudControlConfiguration,
    CloudControlDirs,
    CloudControlConfig,
    UpdateCloudControlYamlConfig,
    CloudControlActualizeConfig,
    ConfigWithSectionsParserCCWorker,
    MysqlCommands,
    RabbitCommands,
    CloudControlMigration,
    SystemServicesRestart,
]

update_pipeline = [
    UpdateCC,
    CloudControlConfiguration,
    UpdateCloudControlYamlConfig,
    CloudControlActualizeConfig,
    CloudControlMigration,
    SupervisorRestartCC,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallCC,
    AppendPath,
    CloudControlRenameConfig,
    MysqlDropCommands,
    CloudControlDropConfig,
    CloudControlDropSupervisorConfig,
    CloudControlDropLogFiles,
    DropRabbitVhost,
    SystemServicesRestart,
]

connections_check_pipeline = [
    DbConnectionCheck,
    KeystoneConnectionCheck,
    RabbitMQConnectionCheck,
]

post_check_pipeline = []
