# coding: utf-8

from __future__ import unicode_literals

import logging
import os

from tionix_autosetup.executors.base import MODULES_CONF_DIR, Dirs
from tionix_autosetup.executors.config import (
    Config, DropConfig, RenameConfig, DropPath, UpdateYamlConfig,
    ActualizeConfig)
from tionix_autosetup.executors.keystone import KeystoneConnectionCheck
from tionix_autosetup.executors.package import (
    Install, Update, Uninstall)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, RunConfiguration)
from tionix_autosetup.helpers import PackageTypesEnum

logger = logging.getLogger(__name__)

dirs = ['/var/log/tionix/monitor']
config_name = 'monitor.yaml'


package_names = {
    PackageTypesEnum.PIP: 'tionix-monitor',
    PackageTypesEnum.RPM: 'python-module-tionix_monitor',
}


class MonitorConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, config_name)
    template_name = 'monitor_config.template'


class UpdateMonitorYamlConfig(UpdateYamlConfig, MonitorConfig):
    pass


class MonitorDirs(Dirs):
    dirs = dirs


class MonitorDropConfig(DropConfig):
    config_name = config_name


class MonitorRenameConfig(RenameConfig):
    config_name = config_name


class MonitorActualizeConfig(ActualizeConfig):
    config_name = config_name


class MonitorDropLogFiles(DropPath):
    paths = dirs


class InstallMonitor(Install):
    package_names = package_names


class UpdateMonitor(Update):
    package_names = package_names


class UninstallMonitor(Uninstall):
    package_names = package_names


class MonitorConfiguration(RunConfiguration):
    module_names = 'tnx_monitor'


pipeline = [
    InstallMonitor,
    MonitorConfiguration,
    MonitorDirs,
    MonitorConfig,
    UpdateMonitorYamlConfig,
    MonitorActualizeConfig,
    SystemServicesRestart,
]

update_pipeline = [
    UpdateMonitor,
    MonitorConfiguration,
    UpdateMonitorYamlConfig,
    MonitorActualizeConfig,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallMonitor,
    MonitorRenameConfig,
    MonitorDropConfig,
    MonitorDropLogFiles,
    SystemServicesRestart,
]

connections_check_pipeline = [
    KeystoneConnectionCheck,
]

post_check_pipeline = []
