# coding: utf-8

from __future__ import unicode_literals

import os

from tionix_autosetup.executors.base import MODULES_CONF_DIR, Dirs
from tionix_autosetup.executors.config import (
    DropPath, DropConfig, RenameConfig, DropSupervisorConfig, Config,
    ConfigWithSectionsParser, UpdateYamlConfig, ActualizeConfig)
from tionix_autosetup.executors.mysql import (
    DbConnectionCheck, MysqlDropCommands, MysqlCommands)
from tionix_autosetup.executors.keystone import KeystoneConnectionCheck
from tionix_autosetup.executors.package import (
    Install, Uninstall, Update)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, Migration, RunConfiguration, SupervisorRestart)
from tionix_autosetup.helpers import PackageTypesEnum

dirs = ['/var/log/tionix/node-control']
config_name = 'node_control.yaml'
module_config_prefix = 'tnx_nc'
tionix_nc_sync_nodes = 'tionix-nc-sync-nodes'
tionix_nc_tracker = 'tionix-nc-tracker'
supervisor_process = '{0} {1}'.format(tionix_nc_sync_nodes, tionix_nc_tracker)


package_names = {
    PackageTypesEnum.PIP: 'tionix-node-control',
    PackageTypesEnum.RPM: 'python-module-tionix_node_control',
}


class NodeControlMigration(Migration):
    module_name = 'tnx_node_control'


class NodeControlConfiguration(RunConfiguration):
    module_names = 'tnx_node_control'


class NodeControlConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, config_name)
    template_name = 'node_control_config.template'


class UpdateNodeControlYamlConfig(UpdateYamlConfig, NodeControlConfig):
    pass


class NodeControlActualizeConfig(ActualizeConfig):
    config_name = config_name


class UpdateConfigNodeControl(ConfigWithSectionsParser):
    config_path = '/etc/supervisor/conf.d/tnx_nc_tracker.conf'
    config_section = 'program:{0}'.format(tionix_nc_tracker)
    config_keys = {
        'user': '{TNX_USERNAME}',
        'group': '{TNX_USERNAME}',
    }


class UpdateConfigSyncNodes(ConfigWithSectionsParser):
    config_path = '/etc/supervisor/conf.d/tnx_nc_sync_nodes.conf'
    config_section = 'program:{0}'.format(tionix_nc_sync_nodes)
    config_keys = {
        'user': '{TNX_USERNAME}',
        'group': '{TNX_USERNAME}',
    }


class NodeControlDirs(Dirs):
    dirs = dirs


class NodeControlDropConfig(DropConfig):
    config_name = config_name


class NodeControlDropSupervisorConfig(DropSupervisorConfig):
    module_config_prefix = module_config_prefix


class NodeControlRenameConfig(RenameConfig):
    config_name = config_name


class NodeControlDropLogFiles(DropPath):
    paths = dirs


class InstallNC(Install):
    package_names = package_names


class UpdateNC(Update):
    package_names = package_names


class UninstallNC(Uninstall):
    package_names = package_names


class SupervisorRestartNC(SupervisorRestart):

    @property
    def commands(self):
        return [self.commands_template.format(supervisor_process)]


pipeline = [
    InstallNC,
    NodeControlConfiguration,
    NodeControlDirs,
    NodeControlConfig,
    UpdateNodeControlYamlConfig,
    NodeControlActualizeConfig,
    UpdateConfigNodeControl,
    UpdateConfigSyncNodes,
    MysqlCommands,
    NodeControlMigration,
    SystemServicesRestart,
]

update_pipeline = [
    UpdateNC,
    NodeControlConfiguration,
    UpdateNodeControlYamlConfig,
    NodeControlActualizeConfig,
    NodeControlMigration,
    SupervisorRestartNC,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallNC,
    NodeControlRenameConfig,
    MysqlDropCommands,
    NodeControlDropConfig,
    NodeControlDropSupervisorConfig,
    NodeControlDropLogFiles,
    SystemServicesRestart,
]

connections_check_pipeline = [
    DbConnectionCheck,
    KeystoneConnectionCheck,
]

post_check_pipeline = []
